@@ -10,6 +10,8 @@ from ..scene.scene import Scene, append_scenes
 from ..constants import log_time, log
 
 from . import misc
+
+from .xyz import _xyz_loaders
 from .ply import _ply_loaders
 from .stl import _stl_loaders
 from .dae import _collada_loaders
@@ -17,12 +19,12 @@ from .obj import _obj_loaders
 from .off import _off_loaders
 from .misc import _misc_loaders
 from .gltf import _gltf_loaders
+from .xaml import _xaml_loaders
+from .binvox import _binvox_loaders
 from .assimp import _assimp_loaders
 from .threemf import _three_loaders
 from .openctm import _ctm_loaders
-from .xml_based import _xml_loaders
-from .binvox import _binvox_loaders
-from .xyz import _xyz_loaders
+from .threedxml import _threedxml_loaders
 
 
 try:
@@ -37,13 +39,13 @@ except BaseException as E:
 
 def mesh_formats():
     """
-    Get a list of mesh formats
+    Get a list of mesh formats available to load.
 
     Returns
     -----------
     loaders : list
-        Extensions of available mesh loaders
-        i.e. 'stl', 'ply', etc.
+      Extensions of available mesh loaders,
+      i.e. 'stl', 'ply', etc.
     """
     return list(mesh_loaders.keys())
 
@@ -207,10 +209,6 @@ def load_mesh(file_obj,
                                           file_type=file_type,
                                           resolver=resolver,
                                           **kwargs)
-
-        if util.is_file(file_obj):
-            file_obj.close()
-
         if not isinstance(results, list):
             results = [results]
 
@@ -435,21 +433,20 @@ def load_kwargs(*args, **kwargs):
         data, file_type = kwargs['data'], kwargs['file_type']
         if not isinstance(data, dict):
             data = util.wrap_as_stream(data)
-        k = mesh_loaders[file_type](data,
-                                    file_type=file_type)
+        k = mesh_loaders[file_type](
+            data, file_type=file_type)
         return Trimesh(**k)
 
     def handle_path():
         from ..path import Path2D, Path3D
-        e, v = kwargs['entities'], kwargs['vertices']
-        if 'metadata' in kwargs:
-            m = kwargs['metadata']
+        shape = kwargs['vertices'].shape
+
+        if shape[1] == 2:
+            return Path2D(**kwargs)
+        elif shape[1] == 3:
+            return Path3D(**kwargs)
         else:
-            m = None
-        if v.shape[1] == 2:
-            return Path2D(entities=e, vertices=v, metadata=m)
-        elif v.shape[1] == 3:
-            return Path3D(entities=e, vertices=v, metadata=m)
+            raise ValueError('Vertices must be 2D or 3D!')
 
     def handle_pointcloud():
         return PointCloud(**kwargs)
@@ -629,11 +626,12 @@ mesh_loaders.update(_misc_loaders)
 mesh_loaders.update(_stl_loaders)
 mesh_loaders.update(_ctm_loaders)
 mesh_loaders.update(_ply_loaders)
-mesh_loaders.update(_xml_loaders)
 mesh_loaders.update(_obj_loaders)
 mesh_loaders.update(_off_loaders)
 mesh_loaders.update(_collada_loaders)
 mesh_loaders.update(_gltf_loaders)
+mesh_loaders.update(_xaml_loaders)
+mesh_loaders.update(_threedxml_loaders)
 mesh_loaders.update(_three_loaders)
 mesh_loaders.update(_xyz_loaders)
 
