@@ -4,23 +4,46 @@ material.py
 
 Store visual materials as objects.
 """
+import abc
+import copy
 import numpy as np
 
 from . import color
 from .. import util
 from .. import grouping
+from .. import exceptions
 
 
-class Material(object):
+class Material(util.ABC):
     def __init__(self, *args, **kwargs):
         raise NotImplementedError('material must be subclassed!')
 
     def __hash__(self):
         return id(self)
 
-    @property
+    @abc.abstractproperty
     def main_color(self):
-        raise NotImplementedError('material must be subclassed!')
+        """
+        The "average" color of this material.
+
+        Returns
+        ---------
+        color : (4,) uint8
+          Average color of this material.
+        """
+
+    @property
+    def name(self):
+        if hasattr(self, '_name'):
+            return self._name
+        return 'material_0'
+
+    @name.setter
+    def name(self, value):
+        self._name = value
+
+    def copy(self):
+        return copy.deepcopy(self)
 
 
 class SimpleMaterial(Material):
@@ -53,7 +76,7 @@ class SimpleMaterial(Material):
     def to_color(self, uv):
         return color.uv_to_color(uv, self.image)
 
-    def to_obj(self, tex_name=None, mtl_name=None):
+    def to_obj(self, mtl_name=None):
         """
         Convert the current material to an OBJ format material.
 
@@ -76,38 +99,39 @@ class SimpleMaterial(Material):
         Kd = color.to_float(self.diffuse)[:3]
         Ks = color.to_float(self.specular)[:3]
 
-        if tex_name is None:
-            tex_name = 'material0'
+        # get the name of this material
+        tex_name = self.name
         if mtl_name is None:
             mtl_name = '{}.mtl'.format(tex_name)
 
-        # what is the name of the export image to save
-
-        image_type = self.image.format
-        if image_type is None:
-            image_type = 'png'
-
-        image_name = '{}.{}'.format(
-            tex_name, image_type).lower()
-
         # create an MTL file
-        mtl = '\n'.join(
-            ['# https://github.com/mikedh/trimesh',
-             'newmtl {}'.format(tex_name),
-             'Ka {:0.8f} {:0.8f} {:0.8f}'.format(*Ka),
-             'Kd {:0.8f} {:0.8f} {:0.8f}'.format(*Kd),
-             'Ks {:0.8f} {:0.8f} {:0.8f}'.format(*Ks),
-             'Ns {:0.8f}'.format(self.glossiness),
-             'map_Kd {}'.format(image_name)])
-
-        # save the image texture as bytes in the original format
-        f_obj = util.BytesIO()
-        self.image.save(fp=f_obj, format=image_type)
-        f_obj.seek(0)
+        mtl = ['# https://github.com/mikedh/trimesh',
+               'newmtl {}'.format(tex_name),
+               'Ka {:0.8f} {:0.8f} {:0.8f}'.format(*Ka),
+               'Kd {:0.8f} {:0.8f} {:0.8f}'.format(*Kd),
+               'Ks {:0.8f} {:0.8f} {:0.8f}'.format(*Ks),
+               'Ns {:0.8f}'.format(self.glossiness)]
 
         # collect the OBJ data into files
-        data = {mtl_name: mtl.encode('utf-8'),
-                image_name: f_obj.read()}
+        data = {}
+
+        if self.image is not None:
+            image_type = self.image.format
+            # what is the name of the export image to save
+            if image_type is None:
+                image_type = 'png'
+            image_name = '{}.{}'.format(
+                tex_name, image_type).lower()
+            # save the reference to the image
+            mtl.append('map_Kd {}'.format(image_name))
+
+            # save the image texture as bytes in the original format
+            f_obj = util.BytesIO()
+            self.image.save(fp=f_obj, format=image_type)
+            f_obj.seek(0)
+            data[image_name] = f_obj.read()
+
+        data[mtl_name] = '\n'.join(mtl).encode('utf-8')
 
         return data, tex_name, mtl_name
 
@@ -176,6 +200,83 @@ class SimpleMaterial(Material):
                            baseColorFactor=self.diffuse)
 
 
+class MultiMaterial(Material):
+    def __init__(self, materials=None, **kwargs):
+        """
+        Wrapper for a list of Materials.
+
+        Parameters
+        ----------
+        materials : Optional[List[Material]]
+            List of materials with which the container to be initialized.
+        """
+        if materials is None:
+            self.materials = []
+        else:
+            self._materials = materials
+
+    def __hash__(self):
+        """
+            Provide a hash of the multi material so we can detect
+            duplicates.
+
+            Returns
+            ------------
+            hash : int
+              Xor hash of the contained materials.
+        """
+        hashed = np.bitwise_xor.reduce(
+            [hash(m) for m in self._materials])
+
+        return hashed
+
+    def __iter__(self):
+        return iter(self._materials)
+
+    def __next__(self):
+        return next(self._materials)
+
+    def __len__(self):
+        return len(self._materials)
+
+    @property
+    def main_color(self):
+        """
+        The "average" color of this material.
+
+        Returns
+        ---------
+        color : (4,) uint8
+          Average color of this material.
+        """
+
+    def add(self, material):
+        """
+        Adds new material to the container.
+
+        Parameters
+        ----------
+        material : Material
+            The material to be added.
+        """
+        self._materials.append(material)
+
+    def get(self, idx):
+        """
+        Get material by index.
+
+        Parameters
+        ----------
+        idx : int
+            Index of the material to be retrieved.
+
+        Returns
+        -------
+            The material on the given index.
+        """
+        return self._materials[idx]
+
+
 class PBRMaterial(Material):
     """
     Create a material for physically based rendering as
@@ -197,8 +298,8 @@ class PBRMaterial(Material):
                  roughnessFactor=None,
                  metallicRoughnessTexture=None,
                  doubleSided=False,
-                 alphaMode='OPAQUE',
-                 alphaCutoff=0.5):
+                 alphaMode=None,
+                 alphaCutoff=None):
 
         # (4,) float
         if baseColorFactor is not None:
@@ -230,6 +331,23 @@ class PBRMaterial(Material):
         self.name = name
         self.alphaMode = alphaMode
 
+    def copy(self):
+        # doing a straight deepcopy fails probably due to PIL images
+        kwargs = {}
+        # collect stored values as kwargs
+        for k, v in self.__dict__.items():
+            if v is None:
+                continue
+            if k.startswith('_'):
+                k = k[1:]
+            if hasattr(v, 'copy'):
+                # use an objects explicit copy if available
+                kwargs[k] = v.copy()
+            else:
+                # otherwise just hope deepcopy does something
+                kwargs[k] = copy.deepcopy(v)
+        return PBRMaterial(**kwargs)
+
     def to_color(self, uv):
         """
         Get the rough color at a list of specified UV coordinates.
@@ -306,7 +424,11 @@ def empty_material(color=None):
     material : SimpleMaterial
       Image is a a one pixel RGB
     """
-    from PIL import Image
+    try:
+        from PIL import Image
+    except BaseException as E:
+        return exceptions.ExceptionModule(E)
+
     if color is None or np.shape(color) not in ((3,), (4,)):
         color = np.array([255, 255, 255], dtype=np.uint8)
     else:
@@ -369,7 +491,8 @@ def pack(materials, uvs, deduplicate=True):
     if deduplicate:
         # start by collecting a list of indexes for each material hash
         unique_idx = collections.defaultdict(list)
-        [unique_idx[hash(m)].append(i) for i, m in enumerate(materials)]
+        [unique_idx[hash(m)].append(i)
+         for i, m in enumerate(materials)]
         # now we only need the indexes and don't care about the hashes
         mat_idx = list(unique_idx.values())
     else:
